#ifndef __ScrGL_h
#define __ScrGL_h
#include <gl\gl.h>



class PACKAGE TGLICDDetails : public TObject
{
public:
  __fastcall TGLICDDetails() : TObject() {};
  __fastcall ~TGLICDDetails() {};
  AnsiString FileName;
  AnsiString CompanyName;
  AnsiString FileDescription;
  AnsiString FileVersion;
  bool Uses3dfxStyle;
};

class PACKAGE TGLICDList : public TComponent
{
public:
  __fastcall TGLICDList(TComponent *Owner);
  __fastcall ~TGLICDList();
public:
  __property TStrings *SearchPath = {read=GetSearchPath};
  void __fastcall SetSearchPath(TStrings *s);
  __property TGLICDDetails* Details[int Index] = {read=GetDetail};
  __property int Count = {read=GetCount};
  __property TStrings* Items = {read=GetList};
  int __fastcall FilenameToIndex(AnsiString fn);
protected:
  TStringList *FList;
  TStringList *FSearchPath;
  void __fastcall DeleteList();
  void __fastcall BuildList();
  TStrings* __fastcall GetSearchPath();
  TGLICDDetails* __fastcall GetDetail(int i);
  int __fastcall GetCount();
  TStrings* __fastcall GetList();
};



enum TGLFlag {f_DOUBLEBUFFER, f_DRAW_TO_BITMAP, f_DRAW_TO_WINDOW,
              f_GENERIC_ACCELERATED, f_GENERIC_FORMAT, f_NEED_PALETTE,
              f_NEED_SYSTEM_PALETTE, f_SUPPORT_GDI, f_SUPPORT_OPENGL, f_STEREO,
              f_SWAP_COPY, f_SWAP_EXCHANGE, f_SWAP_LAYER_BUFFERS};

typedef Set<TGLFlag, f_DOUBLEBUFFER, f_SWAP_LAYER_BUFFERS> TGLFlags;

enum TGLPixelType {pt_TYPE_RGBA, pt_TYPE_COLORINDEX};

enum TGLScreenSize {ss512x384,ss640x480,ss800x600,ss1024x768};

typedef void __fastcall (__closure *TGLErrorEvent)(System::TObject* Sender,AnsiString err);

class PACKAGE TScrGL : public TWinControl
{ public:
  __fastcall TScrGL(TComponent *Owner);
  __fastcall ~TScrGL();
__published:
  __property AnsiString GLICD = {read=FGLICD,write=SetGLICD};
  __property short GLColorBits = {read=FGLColorBits,write=SetGLColorBits,default=32};
  __property short GLDepthBits = {read=FGLDepthBits,write=SetGLDepthBits,default=32};
  __property TGLFlags GLFlags = {read=FGLFlags, write=SetGLFlags, nodefault};
  __property TGLPixelType GLPixelType = {read=FGLPixelType, write=SetGLPixelType, default=pt_TYPE_RGBA};
  __property TNotifyEvent OnGLInit = {read=FOnGLInit,write=FOnGLInit,default=NULL};
  __property TNotifyEvent OnGLPaint = {read=FOnGLPaint,write=FOnGLPaint,default=NULL};
  __property TNotifyEvent OnGLDestroy = {read=FOnGLDestroy,write=FOnGLDestroy,default=NULL};
  __property TGLErrorEvent OnGLError = {read=FOnGLError,write=FOnGLError,default=NULL};
  __property TNotifyEvent OnResize = {read=FOnResize,write=FOnResize,default=NULL};
  __property TColor Color = {read=FColor,write=SetColor,default=clNone};
  __property TGLScreenSize ScreenSize3dfx = {read=FScreenSize,write=SetScreenSize};
  __property Visible;
  __property Align;
  __property TabStop;
public:
  void __fastcall GLHandleNeeded();
  __property HGLRC GLHandle = {read=GetGLHandle};
  __property HDC GLCanvasHandle = {read=GetGLCanvasHandle};
  __property bool IsOn = {read=GetIsOn};
protected:
  void __fastcall EnterMode();
  void __fastcall ExitMode();
  void __fastcall CreateParams(TCreateParams &p);
  void __fastcall WndProc(TMessage &msg);
  void __fastcall SetGLColorBits(const short c);
  void __fastcall SetGLDepthBits(const short d);
  void __fastcall SetGLFlags(const TGLFlags f);
  void __fastcall SetGLPixelType(const TGLPixelType t);
  void __fastcall SetColor(const TColor c);
  HGLRC __fastcall GetGLHandle();
  HDC __fastcall GetGLCanvasHandle();
  void __fastcall InvalidateGLHandle();
  void __fastcall GLError(AnsiString s);
  void __fastcall SetGLICD(AnsiString s);
  void __fastcall ClipMouse();
  void __fastcall UnclipMouse();
  void __fastcall SetScreenSize(TGLScreenSize s);
  bool __fastcall GetIsOn();
  int FSuppressedWidth, FSuppressedHeight;
  TAlign FSuppressedAlign;
  bool FIsSuppressed;
  TGLScreenSize FScreenSize;
  AnsiString FGLICD; HINSTANCE FhGLib;
  HDC Fhdc; HGLRC Fhgl;
  bool FIsMouseClipped;
  RECT FOldMouseClip;
  int FOldMouseSpeed;
  short FGLColorBits;
  short FGLDepthBits;
  TGLFlags FGLFlags;
  TGLPixelType FGLPixelType;
  TColor FColor;
  TNotifyEvent FOnGLPaint;
  TNotifyEvent FOnGLInit;
  TNotifyEvent FOnGLDestroy;
  TGLErrorEvent FOnGLError;
  TNotifyEvent FOnResize;
  HWND hwndSubclass;
  bool GiveUpTrying;
public:
  LRESULT __fastcall SubclassFormProc(FARPROC OldWndProc,HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam);
__published:
  __property OnMouseMove;
  __property OnMouseDown;
  __property OnMouseUp;
};





// SUBSTGL OVERVIEW
//
// Dynamic linking is good. In a screensaver we might want to link with opengl32.dll
// for running in the preview in display properties, and with 3dfxgl.dll for
// running full-screen. So how do we achieve dynamic linking? read on...
//
// If we made a call to any of the standard functions (glBegin, SetPixelFormat,
// wglCreateContext, gluLookAt, ...) then BCB would automatically link us with
// OPENGL32.LIB. And that will force us to use OPENGL32.DLL. This is bad.
// Therefore, we only ever make calls to sglBegin, sglSetPixelFormat,
// sglCreateContext, sgluLookAt and so on. Any call that begins with sgl is
// part of SubstGL and will work properly dynamically.
//
// Before making any sgl calls, you must
//   hlib=sglLoadLibrary("3dfxgl.dll");
// which will use the standard conventions for finding DLL paths. After the end, use
//   sglFreeeLibrary(hlib);
//
// These following are the windows calls that you must change over to SubstGL.
// I have listed them here because you might have been fooled by their names
// into thinking that they are unrelated to GL.
//   sglSwapBuffers
//   sglChoosePixelFormat
//   sglDescribePixelFormat
//   sglSetPixelFormat
//
// A MiniGL has been produced for quake and halflife and others, which is smaller
// and implements only some of the functions. It would give you more flexibility
// in your distribution if you too restricted yourself to calls in the MiniGL.
// Control this with Project|Options|Defines, and define SUBST_MINIGLONLY
// Users of your program can put whichever ICD (installable client driver) into
// your directory that they wish. Or you could include some yourself.
//
// If you had made any calls to the glu functions, then this would implictly
// have linked with GLU32.DLL. But this library implicitly links into OPENGL32.DLL.
// Therefore we are not allowed to call any glu functions at all.
// A few sglu functions have been reimplemented below: sgluLookAt, sgluPickMatrix,
// sgluPerspective, sgluUnProject. Their source code was taken from Mesa, a free
// OpenGL implementation, thereby forcing ScrGL to use the GNU public library
// license. If you don't want to be saddled by this license, you must delete
// those functions and figure out workarounds.
//
//
// PROBLEMS? WHAT NEXT?
//
// This code is only partially developed. More functions need to be added.
// I am adding them in an ad-hoc manner, as I go along.
// At the bottom of this file is a list of MiniGL functions that have not
// yet been added. If you attempt to call any of them, you will get a link
// error.
// I've had problems with some functions not working with the MiniGL driver.
// I don't know why, but will try to document those that I find.
// If you have any questions, email ljw1004@cam.ac.uk or visit my web page
// at www.wischik.com/lu
//



HINSTANCE sglLoadLibrary(AnsiString fn);
void sglFreeLibrary(HINSTANCE hlib);
bool sglUses3dfxStyle();



void            APIENTRY sglAlphaFunc(GLenum func, GLclampf ref);
GLboolean       APIENTRY sglAreTexturesResident(GLsizei n, const GLuint *textures, GLboolean *residences);
// glArrayElementEXT
void            APIENTRY sglBegin(GLenum mode);
void            APIENTRY sglBindTexture(GLenum target, GLuint texture);
void            APIENTRY sglBlendFunc(GLenum sfactor, GLenum dfactor);
void            APIENTRY sglClear(GLbitfield mask);
void            APIENTRY sglClearColor(GLclampf red, GLclampf green, GLclampf blue, GLclampf alpha);
void            APIENTRY sglClearDepth(GLclampd depth);
void            APIENTRY sglColor3f(GLfloat red, GLfloat green, GLfloat blue);
void            APIENTRY sglColor3fv(const GLfloat *v);
void            APIENTRY sglColor4f(GLfloat red, GLfloat green, GLfloat blue, GLfloat alpha);
void            APIENTRY sglColor4fv(const GLfloat *v);
void            APIENTRY sglColor4ub(GLubyte red, GLubyte green, GLubyte blue, GLubyte alpha);
void            APIENTRY sglColor4ubv (const GLubyte *v);
// glColorPointerEXT
void            APIENTRY sglCullFace(GLenum mode);
void            APIENTRY sglDepthFunc(GLenum func);
void            APIENTRY sglDepthMask (GLboolean flag);
void            APIENTRY sglDepthRange (GLclampd zNear, GLclampd zFar);
void            APIENTRY sglDisable(GLenum cap); // TEXTURE_2D, BLEND, DEPTH_TEST, CULL_FACE, ALPHA_TEST only
void            APIENTRY sglDisableClientState (GLenum array);
// glDrawArraysEXT
void            APIENTRY sglDrawBuffer (GLenum mode);
void            APIENTRY sglDrawElements (GLenum mode, GLsizei count, GLenum type, const GLvoid *indices);
// glEdgeFlagPointerEXT
void            APIENTRY sglEnable(GLenum cap); // TEXTURE_2D, BLEND, DEPTH_TEST, CULL_FACE, ALPHA_TEST only
void            APIENTRY sglEnableClientState (GLenum array);
void            APIENTRY sglEnd(void);
void            APIENTRY sglFinish(void);
void            APIENTRY sglFlush();
void            APIENTRY sglFrontFace(GLenum mode);
void            APIENTRY sglFrustum(GLdouble left, GLdouble right, GLdouble bottom, GLdouble top, GLdouble zNear, GLdouble zFar);
void            APIENTRY sglGenTextures (GLsizei n, GLuint *textures);
GLenum          APIENTRY sglGetError(void);
void            APIENTRY sglGetFloatv(GLenum pname, GLfloat *params);
void            APIENTRY sglGetIntegerv(GLenum pname, GLint *params);
// glGetPintervEXT
const GLubyte * APIENTRY sglGetString(GLenum name);
// glIndexPointerEXT
void            APIENTRY sglInterleavedArrays (GLenum format, GLsizei stride, const GLvoid *pointer);
GLboolean       APIENTRY sglIsTexture (GLuint texture);
void            APIENTRY sglLoadIdentity(void);
void            APIENTRY sglLoadMatrixf(const GLfloat *m);
void            APIENTRY sglMatrixMode(GLenum mode); // PROJECTION, MODELVIEW only
void            APIENTRY sglMultMatrixd(const GLdouble *m);
void            APIENTRY sglMultMatrixf(const GLfloat *m);
void            APIENTRY sglOrtho(GLdouble left, GLdouble right, GLdouble bottom, GLdouble top, GLdouble zNear, GLdouble zFar);
void            APIENTRY sglPixelStoref (GLenum pname, GLfloat param);
void            APIENTRY sglPixelStorei(GLenum pname, GLint param);
void            APIENTRY sglPolygonMode (GLenum face, GLenum mode);
void            APIENTRY sglPopMatrix(void);
void            APIENTRY sglPrioritizeTextures (GLsizei n, const GLuint *textures, const GLclampf *priorities);
void            APIENTRY sglPushMatrix(void);
void            APIENTRY sglReadBuffer (GLenum mode);
void            APIENTRY sglReadPixels (GLint x, GLint y, GLsizei width, GLsizei height, GLenum format, GLenum type, GLvoid *pixels);
void            APIENTRY sglRotated (GLdouble angle, GLdouble x, GLdouble y, GLdouble z);
void            APIENTRY sglRotatef(GLfloat angle, GLfloat x, GLfloat y, GLfloat z);
void            APIENTRY sglScaled (GLdouble x, GLdouble y, GLdouble z);
void            APIENTRY sglScalef(GLfloat x, GLfloat y, GLfloat z);
void            APIENTRY sglScissor (GLint x, GLint y, GLsizei width, GLsizei height);
void            APIENTRY sglShadeModel(GLenum mode);
void            APIENTRY sglTexCoord2f(GLfloat s, GLfloat t);
void            APIENTRY sglTexCoord2fv(const GLfloat *v);
// glTextCoordPointerEXT
void            APIENTRY sglTexEnvf(GLenum target, GLenum pname, GLfloat param); // MODULATE REPLACE, DECAL
void            APIENTRY sglTexEnvfv (GLenum target, GLenum pname, const GLfloat *params);
void            APIENTRY sglTexImage2D(GLenum target, GLint level, GLint internalformat, GLsizei width, GLsizei height, GLint border, GLenum format, GLenum type, const GLvoid *pixels);
void            APIENTRY sglTexParameterf(GLenum target, GLenum pname, GLfloat param);
void            APIENTRY sglTexParameterfv (GLenum target, GLenum pname, const GLfloat *params);
void            APIENTRY sglTranslated(GLdouble x, GLdouble y, GLdouble z);
void            APIENTRY sglTranslatef(GLfloat x, GLfloat y, GLfloat z);
void            APIENTRY sglVertex2f(GLfloat x, GLfloat y);
void            APIENTRY sglVertex2fv (const GLfloat *v);
void            APIENTRY sglVertex3f(GLfloat x, GLfloat y, GLfloat z);
void            APIENTRY sglVertex3fv(const GLfloat *v);
// glVertexPointerEXT
void            APIENTRY sglViewport(GLint x, GLint y, GLsizei width, GLsizei height);




// STUFF NOT IN THE MINIGL:
#ifndef SUBST_MINIGLONLY
void            APIENTRY sglCallList(GLuint list);
void            APIENTRY sglColor3d(GLdouble red, GLdouble green, GLdouble blue);
void            APIENTRY sglDeleteLists(GLuint list,GLsizei range);
void            APIENTRY sglEndList();
GLuint          APIENTRY sglGenLists(GLsizei range);
void            APIENTRY sglGetDoublev(GLenum pname,GLdouble *params);
void            APIENTRY sglHint(GLenum target, GLenum mode);
void            APIENTRY sglInitNames();
GLboolean       APIENTRY sglIsEnabled(GLenum cap);
void            APIENTRY sglNewList(GLuint list, GLenum mode);
void            APIENTRY sglPolygonOffset(GLfloat factor, GLfloat units);
void            APIENTRY sglPopName();
void            APIENTRY sglPushName(GLuint name);
GLint           APIENTRY sglRenderMode(GLenum mode);
void            APIENTRY sglSelectBuffer(GLsizei size, GLuint *buffer);
void            APIENTRY sglTexCoord3f(GLfloat s, GLfloat t, GLfloat r);
void            APIENTRY sglTexCoord3fv(const GLfloat *v);
void            APIENTRY sglVertex4f(GLfloat x, GLfloat y, GLfloat z, GLfloat w);
void            APIENTRY sglVertex3d(GLdouble x, GLdouble y,GLdouble z);
void            APIENTRY sglPushAttrib(GLbitfield mask);
void            APIENTRY sglPopAttrib();
void            APIENTRY sglVertex2i(GLint x, GLint y);
#endif




// Use these instead of the wgl* functions
HGLRC APIENTRY sglCreateContext(HDC hdc);
BOOL  APIENTRY sglDeleteContext(HGLRC hglrc);
BOOL  APIENTRY sglMakeCurrent(HDC hdc, HGLRC hglrc);
BOOL  APIENTRY sglSwapLayerBuffers(HDC hdc, UINT fuPlanes);

// use these functions instead of the GDI ones
BOOL  APIENTRY sglSwapBuffers(HDC hdc);
int   APIENTRY sglChoosePixelFormat(HDC hdc, CONST PIXELFORMATDESCRIPTOR * ppfd);
int   APIENTRY sglDescribePixelFormat(HDC hdc, int iPixelFormat, UINT nBytes, LPPIXELFORMATDESCRIPTOR ppfd);
BOOL  APIENTRY sglSetPixelFormat(HDC hdc, int iPixelFormat, CONST PIXELFORMATDESCRIPTOR * ppfd);



GLint APIENTRY sgluUnProject(GLdouble winx,GLdouble winy,GLdouble winz,
                            const GLdouble model[16],const GLdouble proj[16],
                            const GLint viewport[4],
                            GLdouble *objx,GLdouble *objy,GLdouble *objz);
void APIENTRY sgluPickMatrix( GLdouble x, GLdouble y,
                             GLdouble width, GLdouble height,
                             const GLint viewport[4] );
void APIENTRY sgluPerspective( GLdouble fovy, GLdouble aspect,
                              GLdouble zNear, GLdouble zFar );
void APIENTRY sgluLookAt( GLdouble eyex, GLdouble eyey, GLdouble eyez,
                         GLdouble centerx, GLdouble centery, GLdouble centerz,
                         GLdouble upx, GLdouble upy, GLdouble upz );
void APIENTRY sgluOrtho2D( GLdouble left, GLdouble right,
                          GLdouble bottom, GLdouble top );


// MINIGL COMMENTS
// Ryan's web site (http://home.bc.rogers.wave.ca/borealis/ryan.html) has a list
// of those calls which are supported by the MiniGL. I have taken the list from
// that.
//
// The following calls are part of the MiniGL but have not been implemented.
// If you try to use them, you will get a linker error.
// sglAreTexturesResident
// sglArrayElementEXT
// sglColor4ub, glColor4ubv
// sglColorPointerEXT
// sglDepthMask
// sglDepthRange
// sglDisableClientState
// sglDrawArraysEXT
// sglDrawBuffer
// sglDrawElements
// sglEdgeFlagPointerEXT
// sglEnableClientState
// sglGenTextures
// sglGetPintervEXT
// sglIndexPointerEXT
// sglIsTexture
// sglInterleavedArrays
// sglPixelStoref
// sglPolygonMode
// sglPrioritize
// sglReadBuffer
// sglReadPixels
// sglRotated
// sglScaled
// sglScissor
// sglTextCoordPointerEXT
// sglTexEnvfv
// sglTexParameterfv
// sglVertex2fv
// sglVertexPointerEXT


#endif